import numpy as np


def lagmat(A: np.array, lags: list, orient: str = 'col') -> np.array:
    """Create array with time-lagged copies of the features/variables

    Args:
        A (np.ndarray): Dataset. One column for each features/variables,
            and one row for each example/observation at a certain time step.
        lags (ist, tuple): Definition what time lags the copies of A should
            have.
        orient (str, Default: 'col'): Information if time series in A are in
            column-oriented or row-oriented

    Assumptions:
        - It's a time-homogenous time series (that's why there is
            no time index)
        - Features/Variables are ordered from the oldest example/observation
            (1st row) to the latest example (last row)
        - Any Missing Value treatment have been done previously.
    """
    # detect negative lags
    if min(lags) < 0:
        raise Exception((
            "Negative lags are not allowed. Only provided integers "
            "greater equal 0 as list/tuple elements"))
    # None result for missing lags
    if len(lags) == 0:
        return None
    # enforce floating subtype
    if not np.issubdtype(A.dtype, np.floating):
        A = np.array(A, np.float32)

    if orient in ('row', 'rows'):
        # row-oriented time series
        if len(A.shape) == 1:
            A = A.reshape(1, -1)
        A = np.array(A, order='C')
        return lagmat_rows(A, lags)

    elif orient in ('col', 'cols', 'columns'):
        # column-oriented time series
        if len(A.shape) == 1:
            A = A.reshape(-1, 1)
        A = np.array(A, order='F')
        return lagmat_cols(A, lags)
    else:
        return None


def lagmat_rows(A: np.array, lags: list):
    # number of colums and lags
    n_rows, n_cols = A.shape
    n_lags = len(lags)
    # allocate memory
    B = np.empty(shape=(n_rows * n_lags, n_cols), order='C', dtype=A.dtype)
    B[:] = np.nan
    # Copy lagged columns of A into B
    for i, l in enumerate(lags):
        # target rows of B
        j = i * n_rows
        k = j + n_rows  # (i+1) * n_rows
        # number cols of A
        nc = n_cols - l
        # Copy
        B[j:k, l:] = A[:, :nc]
    return B


def lagmat_cols(A: np.array, lags: list):
    # number of colums and lags
    n_rows, n_cols = A.shape
    n_lags = len(lags)
    # allocate memory
    B = np.empty(shape=(n_rows, n_cols * n_lags), order='F', dtype=A.dtype)
    B[:] = np.nan
    # Copy lagged columns of A into B
    for i, l in enumerate(lags):
        # target columns of B
        j = i * n_cols
        k = j + n_cols  # (i+1) * ncols
        # number rows of A
        nl = n_rows - l
        # Copy
        B[l:, j:k] = A[:nl, :]
    return B



from lagmat import lagmat
import numpy as np
import numpy.testing as npt


def test1():
    A = np.array([1, 2, 3, 4])
    B = lagmat(A, [1, 2])
    C = np.array([[np.nan, np.nan], [1, np.nan], [2, 1], [3, 2]])
    npt.assert_array_equal(B, C)



import numpy as np
import warnings


def chopnan(X: np.array, lags: list = [], nchop: int = None) -> np.array:
    if lags:
        nchop = max(lags)
    if nchop is None:
        raise Exception("No lags=[1,2,..] nor nchop=? has been provided")
    return X[nchop:, :]


def rmnanrow(X: np.array) -> np.array:
    """Remove all rows that contain a NaN value"""
    warnings.warn((
        "This function 'rmnanrow' is for demonstration purpose only. "
        "You may risk to delete rows due to missing values "
        "that need to be fixed during data cleansing before "
        "feature engineering."))
    idx = np.any(np.isnan(X), axis=1)
    return X[np.logical_not(idx), :]
